from .utils import no_api_key, repeat_test_without_api_key, use_play


def test_api_key():
    from elevenlabs import get_api_key

    assert get_api_key() is not None
    with no_api_key():
        assert get_api_key() is None
    assert get_api_key() is not None


@repeat_test_without_api_key
def test_voices():
    from elevenlabs import voices

    # Test that we can get all voices
    all_voices = voices()
    assert len(all_voices) > 0


@repeat_test_without_api_key
def test_generate():
    from elevenlabs import generate, play

    # Test that we can generate audio
    audio = generate(text="Test voice generation.")
    assert isinstance(audio, bytes) and len(audio) > 0
    if use_play:
        play(audio)


def test_generate_stream():
    from typing import Iterable

    from elevenlabs import generate, stream

    # Test that we can generate audio stream
    audio_stream = generate(text="Test voice streaming.", stream=True)
    assert isinstance(audio_stream, Iterable)

    if use_play:
        audio = stream(audio_stream)
        assert isinstance(audio, bytes) and len(audio) > 0


def test_generate_stream_optimized():
    from typing import Iterable

    from elevenlabs import generate, stream

    # Test that we can generate audio stream
    audio_stream = generate(
        text="Test voice streaming optimized latency.", stream=True, latency=4
    )
    assert isinstance(audio_stream, Iterable)

    if use_play:
        audio = stream(audio_stream)
        assert isinstance(audio, bytes) and len(audio) > 0


def test_generate_with_voice():
    from elevenlabs import generate, play, voices

    # Test voice with name
    audio = generate(text="Test voice with name.", voice="Rachel")
    assert isinstance(audio, bytes) and len(audio) > 0
    if use_play:
        play(audio)

    # Test voice with id
    audio = generate(
        text="Test voice with id.",
        voice="21m00Tcm4TlvDq8ikWAM",
    )
    assert isinstance(audio, bytes) and len(audio) > 0
    if use_play:
        play(audio)

    # Test voice with Voice object
    voice = voices()[2]
    audio = generate(text="Test voice with Voice object.", voice=voice)
    assert isinstance(audio, bytes) and len(audio) > 0
    if use_play:
        play(audio)


def test_generate_multilingual_v1():
    from elevenlabs import generate, play

    # Test multilingual model
    audio = generate(text="Prueba modelo multilingüe.", model="eleven_multilingual_v1")
    assert isinstance(audio, bytes) and len(audio) > 0
    if use_play:
        play(audio)



from __future__ import annotations

from typing import List, Optional

from .base import API, Listable, api_base_url_v1


class Model(API):
    model_id: str
    name: Optional[str] = None
    token_cost_factor: Optional[float] = None
    description: Optional[str] = None


class Models(Listable, API):
    models: List[Model]

    @classmethod
    def from_api(cls) -> Models:
        url = f"{api_base_url_v1}/models"
        response = cls.get(url).json()
        return cls(models=response)

    @property
    def items(self):
        return self.models



from __future__ import annotations

from typing import Optional

from .base import API, api_base_url_v1


class Subscription(API):
    tier: str
    character_count: int
    character_limit: int
    can_extend_character_limit: bool
    allowed_to_extend_character_limit: bool
    next_character_count_reset_unix: int
    voice_limit: int
    professional_voice_limit: int
    can_extend_voice_limit: bool
    can_use_instant_voice_cloning: bool
    can_use_professional_voice_cloning: bool
    currency: Optional[str] = None
    status: str

    @classmethod
    def from_api(cls) -> Subscription:
        url = f"{api_base_url_v1}/user/subscription"
        response = API.get(url).json()
        return cls(**response)


class User(API):
    subscription: Subscription

    @classmethod
    def from_api(cls) -> User:
        url = f"{api_base_url_v1}/user"
        response = cls.get(url).json()
        return cls(**response)



import warnings


def test_history():
    from elevenlabs import History, HistoryItem

    page_size = 5
    # Test that we can get history
    history = History.from_api(page_size=page_size)
    assert isinstance(history, History)

    # Test that we can iterate over multiple pages lazily
    it = iter(history)
    for i in range(page_size * 3):
        try:
            assert isinstance(next(it), HistoryItem)
        except StopIteration:
            warnings.warn("Warning: not enough history items to test multiple pages.")
            break


def test_history_item_delete():
    import time
    from random import randint

    from elevenlabs import History, generate

    # Random text
    text = f"Test {randint(0, 1000)}"
    generate(text=text)  # Generate a history item to delete
    time.sleep(1)
    history = History.from_api(page_size=1)
    history_item = history[0]
    # Check that item matches
    assert history_item.text == text
    history_item.delete()
    # Test that the history item was deleted
    history = History.from_api(page_size=1)
    assert len(history) == 0 or history[0].text != text
